# -*- coding: utf-8 -*-

"""Controller API for `global resources' part.

    Functionality description:

    Let's have the main subcontroller (GlobalResController) available through
    'global-resources' identifier of the root controller and the subcontroller
    handling related commands (GlobalResCmdController) through
    'global-resources_cmd' identifier.

    Luci application is accessible via http://example.com. Global resources
    in the system are Resource 1, Resource 2 and Resource 3.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/global-resources>
       OR <http://example.com/global-resources/>
       - displaying a list of clusters with some information about them.

    2) <http://example.com/global-resources/Resource%201>
       OR <http://example.com/global-resources/Resource%201/>
       - the same list as in case 1) is displayed, but also details to global
         resource `Resource 1' are shown in the lower part of the page.

    3) <http://example.com/global-resources/NonExistingResource>
       OR <http://example.com/clusters/NonExistingResource/>
       OR <http://example.com/clusters/NonExistingResource/something_more>
       - redirection back to base `global resources' page and displaying
         information about the request of non-existing global resource.

    ---

    4) <http://example.com/global-resources_cmd/create>
       - display form for cluster creation.

"""

from tg import flash, request, redirect, expose, tmpl_context, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from repoze.what import predicates

from luci.lib.base import SubController, SubControllerApplyMixin, \
                          ApplyCommands, setApplyCommands
from luci.controllers.decorators import *
from luci.lib.app_strings import Title, FlashMsg
from luci.lib.helpers import relativeUrlList2Str

__all__ = ['GlobalResController', 'GlobalResCmdController']


# Imports into module's namespace.
scheme = app_globals.scheme
cluster_scheme = app_globals.scheme.Cluster
data = app_globals.data
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class GlobalResController(SubController):
    """Subcontroller handling basic requests related with `global resources' part."""

    # TODO: Uncomment this (or replace with more suitable predicate)
    # to apply access control.
    # For some reason this won't work for me, similar problem discussed at:
    # <http://groups.google.com.ar/group/turbogears/browse_thread/thread/28a8b855a850a1e7>
    # allow_only = predicates.not_anonymous()


    @forPageShowWithUrlCorrection('luci.templates.global_res')
    def default(self):
        """Handle simple global resources listing.

        E.g. <http://example.com/global-resources> for situation described
        in the module's doc.

        """
        Title.usePart('GLOBAL_RES')

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, scheme.GLOBAL_RES)
        cmd_url_str = relativeUrlList2Str(scheme.APP_PREFIX,
                                          base2CmdCtrl(scheme.GLOBAL_RES))

        return dict(name=None,
                    base_url=base_url_str,
                    cmd_url=cmd_url_str,
                    apply_cmds=GlobalResApplyCommands)


    @expose()
    def lookup(self, name, *args):
        """Handle global resources listing with details of certain one.

        E.g. <http://example.com/global-resources/Resource%201>
        for situation described in the module's doc.

        The request is dynamically delegated to _CertainGlobalResController
        object.

        Keyword arguments:
            name    Name of the global resource to be displayed in more detail.

        """
        # Check whether required global resource.
        if data.global_res.has_key(name):
            dynamic_ctrl = _CertainGlobalResController(entity_name=name)
            return dynamic_ctrl, args
        else:
            msg = l_('Bad global resource name: %s') % name
            status = 'error'

        flash(msg, status=status)
        redirect(relativeUrlList2Str(tmpl_context.cluster_url,
                                     cluster_scheme.GLOBAL_RES))


class _CertainGlobalResController(SubController):
    """Subcontroller handling global resources listing with details of certain one."""

    @forPageShowWithUrlCorrection('luci.templates.global_res')
    def default(self):
        """Handle global resources listing with details of certain one.

        E.g. <http://example.com/global-resources/Resource%201>
        for situation described in the module's doc.

        """
        Title.usePart('CERTAIN_GLOBAL_RES', self.entity_name)

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, scheme.GLOBAL_RES)
        cmd_url_str = relativeUrlList2Str(scheme.APP_PREFIX,
                                          base2CmdCtrl(scheme.GLOBAL_RES))

        return dict(name=self.entity_name,
                    base_url=base_url_str,
                    cmd_url=cmd_url_str,
                    apply_cmds=GlobalResApplyCommands)


    @expose()
    def lookup(self, *args):
        """Handle wrong URL connected with certain global resource.

        E.g. <http://example.com/global-resources/Resource%201/something>
        for situation described in the module's doc.

        """
        redirect(relativeUrlList2Str(scheme.APP_PREFIX, scheme.GLOBAL_RES,
                                self.entity_name))


#
# COMMANDS HANDLING.
#

class GlobalResApplyCommands(ApplyCommands):
    """Encapsulation of available commands to apply over selected global resource(s)."""

    DELETE = 'cmd_delete'

    def _delete(which):
        global_res = data.global_res
        msg = l_('Deleting selected global resource(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for global_resource in which:
            if global_res.has_key(global_resource):
                del global_res[global_resource]
            else:
                # If Luci's data are consistent, this should never happen.
                msg, status = FlashMsg.INTERNAL_ERROR
                break
        return msg, status, False


    # Mapping.
    cmds = {DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _delete = staticmethod(_delete)


@setApplyCommands(GlobalResApplyCommands)
class GlobalResCmdController(SubController, SubControllerApplyMixin):
    """Subcontroller handling commands related with `global resources' part."""

    # TODO: Uncomment this (or replace with more suitable predicate)
    # to apply access control.
    # For some reason this won't work for me, similar problem discussed at:
    # <http://groups.google.com.ar/group/turbogears/browse_thread/thread/28a8b855a850a1e7>
    # allow_only = predicates.not_anonymous()

    @forImmediateRedirect(allowed_methods = ('GET', 'POST'))
    def default(self, *args, **kwargs):
        """Handle requests not connected with any of defined method."""

        flash(*FlashMsg.BAD_COMMAND_REQUEST)

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, self.base_name)
        return dict(redir_target=base_url_str)


    @forImmediateRedirect(allowed_methods = 'GET')
    def add(self):
        """Handle creation of a new node."""

