# -*- coding: utf-8 -*-
"""The base Controller API."""

from tg import TGController, request, flash, app_globals, tmpl_context

from luci.controllers.decorators import forImmediateRedirect
from luci.lib.app_strings import FlashMsg
from luci.lib.helpers import relativeUrlList2Str
import luci.model as model

__all__ = ['BaseController', 'SubController',
           'SubControllerApplyMixin', 'ApplyCommands', 'setApplyCommands']


class BaseController(TGController):
    """Base class for the root controller.

    The root of the application is used to compute URLs used by this app.

    """

    def __call__(self, environ, start_response):
        """Invoke the Controller."""
        # TGController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']

        request.identity = request.environ.get('repoze.who.identity')
        tmpl_context.identity = request.identity
        return TGController.__call__(self, environ, start_response)


class SubController(object):
    """Base class for subcontrollers.

    Keyword arguments passed on object creation are automatically moved
    inside the object as its attributes.

    """

    def __new__(cls, *args, **kwargs):
        # If the class that invoked this method is directly inherited from
        # this class, set new attributes according to passed keyword arguments
        # and delegate this invocation upwards (omitting the level
        # of SubController class).
        # Then delegate the invocation to superclass.

        if cls.__base__.__name__ == 'SubController':
            for kw, v in kwargs.iteritems():
                setattr(cls, kw, v)
            kwargs.clear
        return super(type(cls), cls).__new__(cls, *args)


#-------------------------------------------------------------------------------
# Mini-framework for definition and usage of `apply command over selected
# entitities' feature.

# Name of the attribute in `commands controller' to contain encapsulation
# of related `commands to apply' (derived from ApplyCommands).
# This attribute should be only set with 'setApplyCommands' class decorator,
# the other uses of this attribute belong to the framework.
ATTR_APPLY_CMD = 'APPLY_CMD'


class ApplyCommands:
    """Base class for object encapsulation of related commands `to apply'.

    See doc-string for SubControllerApplyMixin.
    """

    # Default empty mapping.
    cmds = {}
        

class SubControllerApplyMixin:
    """Mixin that adds 'apply' method to the derived controller.

    Class using this mixin has to be decorated with 'setApplyCommands'
    decorator setting the appropriate encapsulation of related commands.
    This encapsulation should be derived from 'ApplyCommands' class.

    Example definition of such encapsulation of related commands:
        ---
    class EntityApplyCommands(ApplyCommands):
            DELETE = 'cmd_delete'

            def _delete(which):
                ...
                return msg, status, False

            # Mapping.
            cmds = {DELETE: _delete}
        
            _delete = staticmethod(_delete)
        ---
        
    The 'apply' method takes all keyword arguments (taken from URL) and try
        to find a command identifier first (e.g. 'cmd_delete' from example
        above, but there can be more of them).
        Then it prepares a list of identifiers (sort of entity depends
        on the context of use, e.g. nodes) in this manner:
            1) if name argument was specified, than the list consists of one
               item -- this name
            2) otherwise the list consists of identifiers (taken from the rest
               of keyword arguments) converted back to string form (they are
               supposed to be previously converted to HTML form-safe sequences)
        Then it calls associated function ('_delete' from example above),
        passing the mentioned list of identifiers.
        
        Such called function ('_delete' from example above) should return
        2- or 3-tuple (msg, status [, use_referrer]), where:
            msg           Message to be displayed (e.g. 'entity X deleted').
            status        Status argument for tg.flash ('info', 'warning', ...).
            use_referrer  Whether to allow use referrer (if available)
                          for redirection target.
        
    """

    @forImmediateRedirect(allowed_methods = ('GET', 'POST'))
    def apply(self, name=None, **kwargs):
        """Handle applying a command over selected entity(-ies).

        Keyword arguments:
            name    Name of the entity to be used (if GET method is used).
            kwargs  Dict of entities to be used (if POST method is used).

        Return value(s):
            Dict as expected by 'forImmediateRedirect' decorator.

        """
        form_utils = app_globals.form_utils

        cmd_set = False
        # Get intersection of keys of request's parameters and
        # commands defined within corresponding encapsulation of commands.
        # This intersection is expected to have only one member.
        apply_commands = getattr(self, ATTR_APPLY_CMD, ApplyCommands)
        cmds_used = set(kwargs).intersection(apply_commands.cmds)
        if len(cmds_used) == 1:
            cmd_set = True
            cmd = cmds_used.pop()
            kwargs.pop(cmd)

        use_referrer = True
        if not cmd_set:
            # If Luci's templates are consistent, this should never happen.
            msg, status = FlashMsg.INTERNAL_ERROR
        else:
            if name:
                which = [name]
            else:
                which = [form_utils.id2String(s) for s in kwargs.iterkeys()]

            if not which:
                msg, status = FlashMsg.NOTHING_CHOSEN
            else:
                retval = apply_commands.cmds[cmd](which)
                unpacker = lambda msg, status, use_referrer=True: \
                           (msg, status, use_referrer)
                msg, status, use_referrer = unpacker(*retval)

        flash(msg, status=status)
        base_url_str = \
            relativeUrlList2Str(tmpl_context.cluster_url, self.base_name)

        return dict(use_referrer=use_referrer,
                    redir_target=base_url_str)


def setApplyCommands(apply_commands):
    """Class decorator to set the encapsulation of related `commands to apply'.

    To be used for decoration of `commands controller' that derives from
    SubControllerApplyMixin.

    Keyword arguments:
        apply_commands    Encapsulation of related commands `to apply'.

    """

    def decorator(cls):
        """Class binding."""
        # Take and set given encapsulation of related commands as an attribute
        # of the class.
        if issubclass(apply_commands, ApplyCommands):
            setattr(cls, ATTR_APPLY_CMD, apply_commands)
        return cls

    return decorator

